#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>
#include <iostream>
#include <iomanip>
#include <math.h>

using namespace std;

// Scheme for the problem of multiplication of multiple diagonal matrix by dense matrix - OpenMP version
// Spanish Parallel Programming Contest 2016, problem A

//initialize a vector to a constant value
void inicializevector(int n,double *m,double v)
{
    for(int i=0;i<n;i++)
        m[i]=v;
}

//itilialize a vector to random values between to given values
void initializevectordouble(int n,int ld,double *vector,double lv,double uv)
{
    for(int i=0;i<n;i++)
      vector[i*ld]=((int) ((((1.*rand())/RAND_MAX)*(uv-lv)+lv)*1000.))/1000.;
}

//initialize a matrix with a maximum of ND non-zero diagonals
void initializemultidiagonalmatrix(int N,int ND,double *A,double lower_value,double upper_value)
{
  int diagonal,row,column,size;
  for(int i=0;i<ND;i++)
  {
//   do
//   {
         diagonal=(int) (((1.*rand())/RAND_MAX)*(2*N-1))-N+1; //the diagonals of matrix A are randomly generated
         if(diagonal<0) //diagonal in the lower triangular part of the matrix
         {
            row=(-1)*diagonal; //where the diagonal begins
            column=0;
            size=N-row;
         }
         else
         {
            row=0;
            column=diagonal;
            size=N-column;
         }
//     } while(A[row*N+column]==0.);
     initializevectordouble(size,N+1,&A[row*N+column],lower_value,upper_value);
  }
}

//display a matrix, only for debug
void escribirdouble(int n,int m,double *a){
  int i, j;

  for (i = 0; i < n; i++)
  {
    for (j = 0; j < m; j++)
    {  
      cout << setiosflags(ios::fixed) << setprecision(4) <<a[i*m+j]<<" ";
    }
    cout <<endl;
  }
  cout <<endl;
}

//output to be compared with the output of the solution provided by the organization
//only possitions multiple of step are written, to spare disk space
void escribirresult(int n,int m,double *a,int step)
{
  int i,j;

  for(i=0;i<n;i++)
  {
     for(j=0;j<m;j++)
     {
        if(((i*m+j)%step)==0)
        {
          cout << setiosflags(ios::fixed) << setprecision(1) <<a[i*m+j]<<" ";
        }
     }
     cout <<endl;
  }
  cout <<endl;
}

//multiply two matrices, only for debug
void multiply(int n,double *a,double *b,double *c)
{
    for(int i=0;i<n;i++)
    {
       for(int j=0;j<n;j++)
       {
          double sum=0.;
          for(int k=0;k<n;k++)
            sum+=a[i*n+k]*b[k*n+j];
          c[i*n+j]=sum;
       }
    }
}

//compare two matrices, only for debug
void compare(int n,double *a,double *b)
{
    for(int i=0;i<n;i++)
    {
        if(fabs(a[i]-b[i])>0.0000001)
            cout <<"Error en " << i <<endl;
    }
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

//abort with the maximum execution time is reached
static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

//the external function to be parallelized by the contestants
extern void sec(int,double *,double *,double *);

int main(int argc,char *argv[]) {
  int num_problems; //number of problems  
  int N; //number of rows and columns of matrices NxN
  int ND; //number of diagonals of the multidiagonal matrix
  int seed; //seed for random generation  
  int gap; //number of data between each output (only for output)
  double lower_value;  //lower bound for the values in the matrix
  double upper_value;  //upper bound for the values in the matrix
  double *A; //multidiagonal matrix
  double *B; //dense matrix
  double *C; //result matrix
  long long ti,tf,tt=0; //initial, final and total times

  FILE *stats_file = fopen("stats", "w");

//to stablish a time limit
  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(60);  /* time limit */
 
//  MPI_Init(&argc,&argv);
//  MPI_Comm_size(MPI_COMM_WORLD,&np);
//  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);

// The number of test cases is read
// and sent to all the processes
//  if(nodo==0)
//  {
    cin >> num_problems;
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
//  else
//  {
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
  for(int i=0;i<num_problems;i++)
  {
//    if(nodo==0)
//    {
      cin >>N;                   // The first argument is the size of the matrices
      cin >>ND;		// Number of diagonals of the multiaiagonal matrix
      cin >>seed; //seed for the random generation of data to generate the matrices to multiply
      cin >>lower_value;  //lower bound for the values in the matrix
      cin >>upper_value;  //upper bound for the values in the matrix
      cin >>gap; //gap between values of the matrices to write in the output
// Space for the data
      A = new double[N*N];
      B = new double[N*N];
      C = new double[N*N];

      inicializevector(N*N,A,0.);
      inicializevector(N*N,C,0.);
      srand(seed);
      initializemultidiagonalmatrix(N,ND,A,lower_value,upper_value);
      initializevectordouble(N*N,1,B,lower_value,upper_value);
#ifdef DEBUG
// While debugging the matrices are written
      escribirdouble(N,N,A);
      escribirdouble(N,N,B);
      escribirdouble(N,N,C);
#endif


//    }
//    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    sec(N,A,B,C);
//    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
//    if(nodo==0)
//    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;
#ifdef DEBUG
// While debugging the time of each execution is written
      fprintf(stats_file, "%Ld\n", tf-ti);
      double *C1=new double[N*N];
      multiply(N,A,B,C1);
      escribirdouble(N,N,C1);
      compare(N*N,C,C1);
      delete[] C1;
#endif
// The results of each problem are written,
// even for the first problem
      escribirresult(N,N,C,gap);
      delete[] A;
      delete[] B;
      delete[] C;
//    } */
  }
  

//  if(nodo==0)
//  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
//  }
//  MPI_Finalize();
  return 0;
}
