#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <iostream>
#include <iomanip>

using namespace std;

// Scheme for the problem of Maximum and substraction in increassing blocks - CUDA version
// Spanish Parallel Programming Contest 2016, problem D

void initializevector(int n,double *a)
{
  int i;

  for(i=0;i<n;i++)
  {
      a[i]=((1.*rand())/RAND_MAX)*2.-1.;
  }
}

void escribir(int n,double *a){
  int i;

  for (i = 0; i < n; i++)
  {
    cout << a[i] << "  ";
  }
  cout <<endl;
}

//output to be compared with the output of the solution provided by the organization
//only possitions multiple of step are written, to spare disk space
void escribirresult(int n,double *a,int step)
{
  int i;

  for(i=0;i<n;i++)
  {
     if((i%step)==0)
     {
        cout << setiosflags(ios::fixed) << setprecision(6) <<a[i]<<" ";
     }
  }
  cout <<endl;
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

extern void sec(int,double *);

int main(int argc,char *argv[]) {
//  int node,np;
  int num_problems; //number of problems  
  int N; //dimension of the vector
  double *vector;
  int seed; //seed for random generation  
  int gap; //at the output, write one of each gap elements of the vector
  long long ti,tf,tt=0; //initial, final and total times

  FILE *stats_file = fopen("stats", "w");

//to stablish a time limit
  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(60);  /* time limit */
 
//  MPI_Init(&argc,&argv);
//  MPI_Comm_size(MPI_COMM_WORLD,&np);
//  MPI_Comm_rank(MPI_COMM_WORLD,&node);

// The number of test cases is read
// and sent to all the processes
//  if(node==0)
//  {
    cin >> num_problems;
//    MPI_Bcast(&num_problems,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
//  else
//  {
//    MPI_Bcast(&num_problems,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
  for(int i=0;i<num_problems;i++)
  {
//    if(node==0)
//    {
      cin >>N;                   // The first argument is the size of the vector
      cin >>seed; //seed for the random generation of data
      cin >>gap;  //number of elements between writings, at the output
     
// Space for the data
      vector = new double[N];

      srand(seed);
      initializevector(N,vector);
#ifdef DEBUG
// While debugging the vector is written
      escribir(N,vector);
#endif

//    }
//    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    sec(N,vector);
//    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
//    if(node==0)
//    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;
#ifdef DEBUG
// While debugging the time of each execution is written
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
#ifdef DEBUG
// While debugging the vector is written
      escribir(N,vector);
#endif
     escribirresult(N,vector,gap);
     delete[] vector;
//    } 
  }
  
//  if(node==0)
//  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
//  }
//  MPI_Finalize();
  return 0;
}
