#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>
#include <iostream>
#include <iomanip>
#include <omp.h>
#include <mpi.h>

using namespace std;

// Scheme for the problem of Game of life with neighborhood without zeros - MPI+OpenMP version
// Spanish Parallel Programming Contest 2016, problem F

void initialize(int n,int *a,int perc0,int uv)
{
  int i,temp;

//generate values between 1 and maximum value
  for(i=0;i<n*n;i++)
  {
      a[i]=(int) (((1.*rand())/RAND_MAX)*uv+1);
  }
//put zero values according with the established percentage
  for(i=0;i<n*n;i++)
  {
      temp=(int) (((1.*rand())/RAND_MAX)*100);
      if(temp<=perc0)
         a[i]=0;
  }
}

void escribir(int n,int *a){
  int i, j;

  for (i = 0; i < n; i++)
  {
    for (j = 0; j < n; j++)
    {  
      cout << a[i*n+j]<<" ";
    }
    cout <<endl;
  }
  cout <<endl;
}

void escribirresult(int n,int *m)
{
   for(int i=0;i<n;i++)
     cout << m[i]<<" ";
   cout <<endl;
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

extern void sec(int,int *,int,int *,int,int);

int main(int argc,char *argv[]) {
  int num_problems; //number of problems  
  int N; //size of the board, NxN
  int *board;
  int ni; //number of iterations
  int seed; //seed for random generation
  int perc0; //percentage of zeros  
  int upper_value;  //maximum value for the generation of values (between 1 and upper_value)
  long long ti,tf,tt=0; //initial, final and total times
  int *num1; //array to store the number of 1 after each iteration
  int np,nodo; //for number of processes and identifier of the process

  FILE *stats_file = fopen("stats", "w");

//to stablish a time limit
  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(60);  /* time limit */
 
  MPI_Init(&argc,&argv);
  MPI_Comm_size(MPI_COMM_WORLD,&np);
  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);

// The number of test cases is read
// and sent to all the processes
  if(nodo==0)
  {
    cin >> num_problems;
    MPI_Bcast(&num_problems,1,MPI_INT,0,MPI_COMM_WORLD);
  }
  else
  {
    MPI_Bcast(&num_problems,1,MPI_INT,0,MPI_COMM_WORLD);
  }
  for(int i=0;i<num_problems;i++)
  {
    if(nodo==0)
    {
      cin >>N;                   // The first argument is the size of the board
      cin >> ni; //number of iterations
      cin >>seed; //seed for the random generation of data
      cin >>perc0;  //percentage of zeros in the board
      cin >>upper_value;  //maximum value to generate in the board
// Space for the data
      board = new int[N*N];
      num1=new int[ni];

      srand(seed);
      initialize(N,board,perc0,upper_value);
#ifdef DEBUG
// While debugging the matrix is written
      escribir(N,board);
#endif

    }
    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    sec(N,board,ni,num1,np,nodo);
    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
#ifdef DEBUG
// While debugging the matrix is written
      escribir(N,board);
#endif
    if(nodo==0)
    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;
#ifdef DEBUG
// While debugging the time of each execution is written
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
      escribirresult(ni,num1);
     delete[] board;
     delete[] num1; 
    } 
  }
  
  if(nodo==0)
  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
  }
  MPI_Finalize();
  return 0;
}
